# Scan log files of ROOL Build system
# (C) 2022 Herbert zur Nedden

# Pfad, wo meine Repositories liegen
$BasePath = $PSScriptRoot
# Pfad zur Summary-Datei
$Summary  = $BasePath + "\Summary.txt"

# Betun eines Repositories
Function DoOne($base) {

	# Blabla...
	echo $base
	
	# Formatierhilfe
	$o = ""

	# Pfad zu den Logs...
	$path = $BasePath + "\" + $base + "\RiscOS\BuildSys\Logs\"

	# Wenn das Logverzeichnis nicht da ist (sprich ein noch ungenutztes Repository) bin ich fertig
	if (!(Test-Path $path)) { return }

	# Alte verdichtete Logs werfe ich weg
	Remove-Item ($path + "\*.txt")
	# Hole die Liste der Logdateien
	# Während ein Log geschrieben wird, hat es noch die Endung ",ffd" - die will ich nicht...
	$files = Get-ChildItem -path $path -Exclude "*,ffd"

	# Wir betun nun alle dateien nacheinander
	foreach ($f in $files) {
		# Das verdichtete Log bekommt denselben Namen wie das Original plus ein ".txt"
		$t = $f.versioninfo.filename + ".txt"
		#T Blabla
		echo ($f.name + " -> " + $t)
		# Bei 'error' muss ich Dinge wie 'c.error', 'error.o' oder 'reporterror' et al übergehen; und auch die Warnung einer definierten aber nicht-genutzten Variable, da das häufig vorkommt...
		# Log einlesen in eine Variable
		$d = get-content $f
		# 'warnings' im Plural als auch '1 warning' mit Leerzeichen, Komma oder Zeilenanfang davor brauche ich nicht...
		$d = $d -replace 'warnings','wrnings'
		$d = $d -replace '( [0-1]) warning','$1 wrning'
		$d = $d -replace '(,[0-1]) warning','$1 wrning'
		$d = $d -replace '(^v) warning','$1 wrning'
		# dito für 'errors'
		$d = $d -replace 'errors','erors'
		$d = $d -replace '( [0-1]) error','$1 eror'
		$d = $d -replace '(,[0-1]) error','$1 eror'
		$d = $d -replace '(^[0-1]) error','$1 eror'
		# Quellcodeobjekte wie reporterror, error.c, c.error etc. filter ich mal weg
		$d = $d -replace '([a-z_\.])error', '$1eror'
		$d = $d -replace 'error([a-z_\.])', 'eror$1'
		# Warnung mit Error im Text
		$d = $d -replace "possible error",'possible eror';
		# Und diese Warnung kommt mir zu oft vor aber ist nun wirklich uninteressant...
		
		$d = $d -replace 'Warning: (variable .* declared but not used)', 'Wrning: $1'
		$d = $d -replace 'Warning: (static .* declared but not used)', 'Wrning: $1'
		$d = $d -replace 'Warning: (unused earlier static declaration of )', 'Wrning: $1'
		$d = $d -replace 'Warning: (Deprecated instruction .STM with PC in register list)', 'Wrning: $1'
		$d = $d -replace 'Warning: (Deprecated instruction .LDM with LR and PC in register list)', 'Wrning: $1'
		$d = $d -replace 'Warning: (LDM or STM of single register is probably slower than LDR or STR)', 'Wrning: $1'
		$d = $d -replace "Warning: (.*: cast of 'int' to differing enum)", 'Wrning: $1'
		$d = $d -replace 'Warning: (<cast>: cast between function pointer and non-function object)', 'Wrning: $1'
		$d = $d -replace 'Warning: (Instruction not supported on targeted CPU)', 'Wrning: $1'
		$d = $d -replace 'Warning: (May trigger StrongARM MSR bug if instruction is not idempotent)', 'Wrning: $1'
		
		# Aus der Zeile 'Started ROOL.xxx' das Paket 'xxx' holen
		$p = ($d | select-string -pattern '^Started ROOL') -replace '(.* )(ROOL\.[a-z0-9]*)( .*)','$2'
		# ... aus der 'Starting phase yyy'-Zeile den Schritt 'yyy' heraussammeln... 
		$s = ($d | select-string -pattern '^Starting phase') -replace '(.*Starting phase )(.*)( .*)','$2'
		# Die Zeilen, die (noch) 'error' bzw. 'warning' enthalten abgreifen
		$e = $d | select-string -pattern 'error'
		$w = $d | select-string -pattern 'warning'
		# Nun bastel ich mir die Summary-Meldung zusammen - mittels substring-Funktion sorge ich für Spaltentreue (der Dateiname in $f.name hat eine feste Länge)
		# Name des Repositories und der Name des Logs
		$m = ($base+"       ").substring(0,8) + " -> " + $f.name + ": "
		# Namen des Pakets aus dem 'Started ROOL' abgreifen und auf Länge bringen
		$m = $m + ($p + "                  ").substring(0,20)
		# Namen des Schritts aus dem 'Starting phase' abgreifen und auf Länge bringen
		$m = $m + ($s + "                  ").substring(0,15)
		# Anzahl der Fehler anhängen (bei 0 Fehlern statt der '0' einen Punkt, damit das besser zu sehen ist)
		if ($e.count -eq 0) { $m = $m + " Errors    ." }
		else                { $m = $m + " Errors "   + ("      "+$e.count).substring(("      "+$e.count).length-4) }
		# Dito für die Warnungen
		if ($w.count -eq 0) { $m = $m + " Warnings     ." }
		else                { $m = $m + " Warnings " + ("      "+$w.count).substring(("      "+$w.count).length-5) }
		# Den Pfad zum verkürzten Log hängen wir mal an, damit leicht öffenbar
		$m = $m + "  " + $t
		# Die Summary-Meldung in die Summary-Datei schreiben
		if ($o -ne $p) {
			"" | Out-File $Summary -Append
			$o = $p
		}
		$m | Out-File $Summary -Append
		# In den Log-Extrakt auch die Summary-Zeile packen
		$m | Out-File $t
		"" | Out-File $t -Append
		# Nach einer Überschrift kommen die Error-Botschaften in den Log-Extrakt
		"------------------------------------------------------------" | Out-File $t -Append
		"- Errors: " | Out-File $t -Append
		"------------------------------------------------------------" | Out-File $t -Append
		$e | Out-File $t -Append
		"" | Out-File $t -Append
		# Nach einer Überschrift kommen die Warning-Botschaften in den Log-Extrakt
		"------------------------------------------------------------" | Out-File $t -Append
		"- Warnings " | Out-File $t -Append
		"------------------------------------------------------------" | Out-File $t -Append
		$w  | Out-File $t -Append
		"" | Out-File $t -Append
	}
	"" | Out-File $Summary -Append
}

# Hauptprogramm beginnt hier

# Summary-Datei löschen wir
"" | Out-File $Summary
Remove-Item ($Summary)

# Einlesen der Repository-Verzeichnisse
$dirs = Get-ChildItem -Path $BasePath -Directory 

# Jedes Repository betun wir nun
foreach ($d in $dirs) {
	DoOne $d.name
}
